(function() {
  var Parse, util;

  util = require("util");

  Parse = (function() {

    function Parse(application_id, master_key) {
      var request;
      this.application_id = application_id;
      this.master_key = master_key;
      this.api_protocol = request = require('request');
      this.api_host = "api.parse.com";
      this.api_port = 443;
    }

    Parse.prototype.parseRequest = function(method, path, data, callback) {
      var auth, headers, msg, options, parts, proxy, req;
      auth = "Basic " + new Buffer(this.application_id + ":" + this.master_key).toString("base64");
      headers = {
        Authorization: auth,
        Connection: "Keep-alive"
      };
      options = {
        method: method,
        uri: "https://api.parse.com:443" + path,
        headers: headers,
        json: true
      };
      switch (method) {
        case "GET":
          if (data) options.qs = data;
          break;
        case "POST":
        case "PUT":
          options.body = JSON.stringify(data);
          options.headers["Content-type"] = "application/json";
          options.headers["Content-length"] = options.body.length;
          break;
        case "DELETE":
          options.headers["Content-length"] = 0;
          break;
        default:
          throw new Error("Unknown method, \"" + method + "\"");
      }
      if (process.env.http_proxy) {
        proxy = require('./proxy');
        parts = process.env.http_proxy.replace('http://', '').split(':');
        options.agent = proxy.createAgent(parts[0], parts[1]);
      }
      msg = "[PARSE][" + method + "] Calling " + options.uri + "%s ->\n\t%s";
      console.log(msg, (options.agent ? ' with proxy' : ''), (data ? util.inspect(data) : ''));
      try {
        return req = this.api_protocol(options, function(error, res, body) {
          var err;
          if (!callback) {
            callback = function(err, body) {
              return {};
            };
          }
          if (error) {
            console.log("[PARSE][ERROR] " + error);
            return callback(error);
          }
          if (res.statusCode < 200 || res.statusCode >= 300) {
            msg = "HTTP error " + res.statusCode;
            console.log("[PARSE][ERROR] " + msg);
            err = new Error(msg);
            err.arguments = arguments;
            err.type = res.statusCode;
            err.options = options;
            err.body = body;
            return callback(err);
          }
          console.log("[PARSE][OK] " + (util.inspect(body).replace("\n", " ")));
          return callback(null, body);
        });
      } catch (exc) {
        return callback(exc);
      }
    };

    Parse.prototype.insert = function(className, object, callback) {
      return this.parseRequest.call(this, "POST", "/1/classes/" + className, object, callback);
    };

    Parse.prototype.list = function(className, callback) {
      return this.parseRequest.call(this, "GET", "/1/classes/" + className, null, callback);
    };

    Parse.prototype.find = function(className, query, callback) {
      if (typeof query === "string") {
        return this.parseRequest.call(this, "GET", "/1/classes/" + className + "/" + query, null, callback);
      } else {
        return this.parseRequest.call(this, "GET", "/1/classes/" + className, {
          where: JSON.stringify(query)
        }, callback);
      }
    };

    Parse.prototype.update = function(className, objectId, object, callback) {
      return this.parseRequest.call(this, "PUT", "/1/classes/" + className + "/" + objectId, object, callback);
    };

    Parse.prototype["delete"] = function(className, objectId, callback) {
      return this.parseRequest.call(this, "DELETE", "/1/classes/" + className + "/" + objectId, null, callback);
    };

    Parse.prototype.now_id = function() {
      var jsonDate, now;
      now = new Date();
      jsonDate = now.toJSON();
      return jsonDate;
    };

    Parse.prototype.noop = function(className, object, callback) {
      return callback(null, 'NOOP:' + JSON.stringify(object));
    };

    return Parse;

  })();

  module.exports = Parse;

}).call(this);
